/*
 * USE - UML based specification environment
 * Copyright (C) 1999-2004 Mark Richters, University of Bremen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// $Id: RealValue.java 61 2008-04-11 11:52:15Z opti $

package org.tzi.use.uml.ocl.value;

import org.tzi.use.uml.ocl.type.TypeFactory;

/**
 * Real values.
 *
 * @version     $ProjectVersion: 0.393 $
 * @author  Mark Richters
 * @see     IntegerValue
 */
public final class RealValue extends Value {
    double fValue;
    
    public RealValue(double d) {
        super(TypeFactory.mkReal());
        fValue = d;
    }

    public double value() {
        return fValue;
    }

    public String toString() {
        return Double.toString(fValue);
    }

    public boolean equals(Object obj) {
        if (obj == this )
            return true;
        else if (obj instanceof RealValue )
            return ((RealValue) obj).fValue == fValue;
        else if (obj instanceof IntegerValue )
            return ((IntegerValue) obj).fValue == fValue;
        return false;
    }

    public int hashCode() {
        // Note: this must be the same hash code as for IntegerValue
        // if we want to treat, e.g., 1.0 equal to 1 in collections.
        return new Double(fValue).hashCode();
    }

    public int compareTo(Object o) {
        if (o == this )
            return 0;
        if (o instanceof IntegerValue ) {
            int val2 = ((IntegerValue) o).fValue;
            return ( fValue < val2 ? -1 : ( fValue == val2 ? 0 : +1) );
        } else if (o instanceof RealValue ) {
            double val2 = ((RealValue) o).fValue;
            return new Double(fValue).compareTo(new Double(val2));
        } else if (o instanceof UndefinedValue ) {
            return +1;
        } else
            return toString().compareTo(o.toString());
    }
}
